/*
  S12232ZA_LEVEL_METER6.ino

  This sketch is for a level meter curcuit controlled by Arduino Uno.
  
  This sketch was developed by Synapse(Hiroshi Tanigawa) in 2013. This sketch is originally
  distributed at Team Schaft's Homepage. <http://www3.big.or.jp/~schaft/hardware>
  
  This sketch is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.
  
  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this sketch.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <TimerOne.h>
#include <EEPROM.h>
#include <MGLCD.h>
#include "kyoto.h"
#include "L.h"
#include "R.h"
#include "meter.h"
#include "miku0.h"
#include "miku1.h"
#include "miku2.h"
#include "miku3.h"
#include "miku4.h"
#include "miku5.h"
#include "miku6.h"
#include "miku7.h"
#include "miku8.h"
#include "logo0.h"
#include "logo1.h"
#include "logo2.h"
#include "logo3.h"
#include "logo4.h"
#include "logo5.h"
#include "logo6.h"
#include "logo7.h"

MGLCD_serial MGLCD(&Serial,500000);
volatile unsigned char TimerCnt=0;

void timerIsr()
{
  TimerCnt++;
} // timerIsr

void PrintSlowly(const char *p)
{
  while(*p!='\0') {
    MGLCD.print(*(p++));
    delay(100);
  } // while
} // PrintSlowly

void opening()
{
  MGLCD.ClearScreen();
  for(int i=90; i>=0; i--) {
    MGLCD.DrawBitmap(BMP_kyoto,i,1,MGLCD_ROM);
    MGLCD.Line(i,0,i,31,0);
    delay(1);
  } // int i
  
  MGLCD.SetInvertMode(MGLCD_INVERT);
  MGLCD.Locate(7,0); PrintSlowly(" PHONE METER ");
  MGLCD.SetInvertMode(MGLCD_NON_INVERT);
  MGLCD.Locate(7,1); PrintSlowly("This software");
  MGLCD.Locate(7,2); PrintSlowly("was designed");
  MGLCD.Locate(7,3); PrintSlowly("in Kyoto.");
  
  delay(3000);
  MGLCD.ClearScreen();  
} // opening

int dB(unsigned x)
{
  int result;
  unsigned level;
  
  result=0;
  level=1;
  while(true) {
    if(x<(level*18U)>>4) {
      return result;
    } else if(x<(level*20U)>>4) {
      return result+1;
    } else if(x<(level*23U)>>4) {
      return result+2;
    } else if(x<(level*25U)>>4) {
      return result+3;
    } else if(x<(level*29U)>>4) {
      return result+4;
    } else if(x<level*2) {
      return result+5;
    } else {
      result+=6;
      level<<=1;
    }
  } // while
} // dB

void setup()
{
  // Initialize LCD
  while(MGLCD.Reset());
  opening();
} // setup

void LevelMeter1()
{
  const int divisor   =  8;
  const int WaveWidth = 53;

  int wave[2][WaveWidth];
  int WaveMin[2];
  int WaveMax[2];
  unsigned char CurrentCnt;
  unsigned char LastCnt;

  unsigned char LastBtn;
  unsigned char CurrentBtn;

  // Initialize Timer1
  Timer1.initialize(50000); // Set length 50ms
  Timer1.attachInterrupt(timerIsr);
  LastCnt=TimerCnt;

  // Initialize screen
  MGLCD.ClearScreen();
  MGLCD.Locate( 0,1); MGLCD.print('L');
  MGLCD.Locate(10,1); MGLCD.print('R');  

  CurrentBtn=MGLCD_BUTTON_OFF;
  do {
    LastBtn=CurrentBtn;
    // Wait
    do {
      CurrentCnt=TimerCnt; 
    } while(CurrentCnt==LastCnt);
    LastCnt=CurrentCnt;
  
    // Get waves
    for(int i=0; i<WaveWidth; i++) {    
      wave[0][i]=(analogRead(A0)-512)/divisor;
      wave[1][i]=(analogRead(A1)-512)/divisor;
    } // for i

    // Clip wave and calculate WaveMin / WaveMax
    WaveMin[0]=WaveMin[1]= 10000;
    WaveMax[0]=WaveMax[1]=-10000;  
    for(int i=0; i<WaveWidth; i++) {    
      for(int j=0; j<2; j++) { 
        if(wave[j][i]> 14) wave[j][i]= 14;
        if(wave[j][i]<-14) wave[j][i]=-14;
        if(wave[j][i]<WaveMin[j]) WaveMin[j]=wave[j][i];
        if(wave[j][i]>WaveMax[j]) WaveMax[j]=wave[j][i];
      } // for j
    } // for j
  
    // Display wave
    for(int i=0; i<WaveWidth; i++) {
      MGLCD.Line( 6+i,0, 6+i,28,0);
      MGLCD.Line(66+i,0,66+i,28,0);
    
      if(i>0) {
        MGLCD.Line( 5+i,14-wave[0][i-1], 6+i,14-wave[0][i]);
        MGLCD.Line(65+i,14-wave[1][i-1],66+i,14-wave[1][i]);      
      } // if
    } // for  
  
    // Display bar graphs
    for(int i=0; i<2; i++) {
      int val;
    
      val=(WaveMax[i]-WaveMin[i])*2;
      if(val>54) val=54;
      MGLCD.FillRect(6+i*60,29,6+i*60+val,31);
      MGLCD.FillRect(7+i*60+val,29,59+i*60,31,0);
    } // for i
    
    CurrentBtn=MGLCD.ReadButton(0);
  } while(CurrentBtn==MGLCD_BUTTON_ON || LastBtn==MGLCD_BUTTON_OFF);  
 
  // Terminate Timer1
  Timer1.detachInterrupt();  
} // LevelMeter1

void LevelMeter2()
{
  const int WaveWidth = 100;

  int BmpWidth;
  int MaxBarWidth;
  unsigned wave[2][WaveWidth];
  unsigned WaveMin[2];
  unsigned WaveMax[2];
  int BarWidth;
  int PeakVal[2]={0,0};
  int PeakCnt[2]={0,0};

  unsigned char LastBtn;
  unsigned char CurrentBtn;

  // Initialize variables
  BmpWidth=pgm_read_byte(BMP_L);
  MaxBarWidth=122-(BmpWidth+2);
  
  // Initialize screen
  MGLCD.ClearScreen();
  MGLCD.DrawBitmap(BMP_L,0, 2,MGLCD_ROM);
  MGLCD.DrawBitmap(BMP_R,0,18,MGLCD_ROM);
  
  CurrentBtn=MGLCD_BUTTON_OFF;
  do {
    LastBtn=CurrentBtn;
    
    // Get waves
    for(int i=0; i<WaveWidth; i++) {    
      wave[0][i]=analogRead(A0);
      wave[1][i]=analogRead(A1);
    } // for i

    // Calculate WaveMin / WaveMax
    WaveMin[0]=WaveMin[1]=10000;
    WaveMax[0]=WaveMax[1]=0; 
    for(int i=0; i<2; i++) { 
      for(int j=0; j<WaveWidth; j++) {    
        if(wave[i][j]<WaveMin[i]) WaveMin[i]=wave[i][j];
        if(wave[i][j]>WaveMax[i]) WaveMax[i]=wave[i][j];
      } // for i
    } // for j
  
    // Draw bar graphs
    for(int i=0; i<2; i++) {
      BarWidth=(dB(unsigned(WaveMax[i]-WaveMin[i]))-10)*3;
      if(BarWidth<0) BarWidth=0;
      if(BarWidth>MaxBarWidth) BarWidth=MaxBarWidth;
    
      if(BarWidth>=PeakVal[i]) {
        PeakVal[i]=BarWidth;
        PeakCnt[i]=0;
      } // if

      if(BarWidth>0          ) MGLCD.FillRect(BmpWidth+2,i*16+1,BmpWidth+2+BarWidth-1,i*16+14,1);
      if(BarWidth<MaxBarWidth) MGLCD.FillRect(BmpWidth+2+BarWidth,i*16+1,121,i*16+14,0);
    
      if(PeakVal[i]>2          ) MGLCD.FillRect(BmpWidth+2+PeakVal[i]-3,i*16+1,BmpWidth+2+PeakVal[i]-1,i*16+14,1);  
      if(PeakVal[i]<MaxBarWidth) MGLCD.FillRect(BmpWidth+2+PeakVal[i],i*16+1,121,i*16+14,0);

      if(PeakVal[i]>0) {
        PeakCnt[i]++;
        if(PeakCnt[i]>=8) {
          PeakVal[i]--;
          PeakCnt[i]=0;
        } // if
      } // if
    } // for i
    
    CurrentBtn=MGLCD.ReadButton(0);
  } while(CurrentBtn==MGLCD_BUTTON_ON || LastBtn==MGLCD_BUTTON_OFF); 
} // LevelMeter2

void LevelMeter3()
{
  const int WaveWidth = 100;
  const int AngleNum  =  37;
  const int DegPerDB  =   3;
  
  unsigned char x0[AngleNum];
  unsigned char y0[AngleNum];
  unsigned char x1[AngleNum];
  unsigned char y1[AngleNum];
  unsigned wave[2][WaveWidth];
  unsigned WaveMin[2];
  unsigned WaveMax[2];
  int LastVal[2]={-1,-1};

  unsigned char LastBtn;
  unsigned char CurrentBtn;
  
  // Initialize variables
  for(int i=0; i<AngleNum; i++) {
    x0[i]=round(29+26*sin((i-(AngleNum-1)/2)*DegPerDB/180.0*PI));
    y0[i]=round(32-26*cos((i-(AngleNum-1)/2)*DegPerDB/180.0*PI));
    x1[i]=round(29+ 8*sin((i-(AngleNum-1)/2)*DegPerDB/180.0*PI));
    y1[i]=round(32- 8*cos((i-(AngleNum-1)/2)*DegPerDB/180.0*PI));    
  } // for i
  
  // Initialize screen
  MGLCD.ClearScreen();
  for(int i=0; i<2; i++) {
    MGLCD.DrawBitmap(BMP_meter,i*63,0);
    for(int j=0; j<AngleNum; j++) {
      MGLCD.Line(i*63+x0[j],y0[j],i*63+x1[j],y1[j],0);
    } // for j
  } // for i
  
  CurrentBtn=MGLCD_BUTTON_OFF;
  do {
    LastBtn=CurrentBtn;

    // Get waves
    for(int i=0; i<WaveWidth; i++) {    
      wave[0][i]=analogRead(A0);
      wave[1][i]=analogRead(A1);
    } // for i

    // Calculate WaveMin / WaveMax
    WaveMin[0]=WaveMin[1]=10000;
    WaveMax[0]=WaveMax[1]=0; 
    for(int i=0; i<2; i++) { 
      for(int j=0; j<WaveWidth; j++) {    
        if(wave[i][j]<WaveMin[i]) WaveMin[i]=wave[i][j];
        if(wave[i][j]>WaveMax[i]) WaveMax[i]=wave[i][j];
      } // for i
    } // for j
  
    // Draw meters
    for(int i=0; i<2; i++) {
      int val;
    
      val=dB(WaveMax[i]-WaveMin[i])-10;
      if(val<0) val=0;
      if(val>=AngleNum) val=AngleNum-1;
    
      if(val!=LastVal[i]) {
        if(LastVal[i]>=0) {
          MGLCD.Line(i*63+x0[LastVal[i]],y0[LastVal[i]],i*63+x1[LastVal[i]],y1[LastVal[i]],0);
        } // if
        MGLCD.Line(i*63+x0[val],y0[val],i*63+x1[val],y1[val],1);
        LastVal[i]=val;
      } // if
    } // for i
    
    CurrentBtn=MGLCD.ReadButton(0);
  } while(CurrentBtn==MGLCD_BUTTON_ON || LastBtn==MGLCD_BUTTON_OFF);
} // LevelMeter3

void LevelMeter4()
{
  const int divisor = 4;
  const int WaveWidth = 100;
  const int LogoWidth =  20;
  const int LogoHeight =112;

  unsigned char * PROGMEM pLogo[8];
  int wave[WaveWidth];
  int WaveMin;
  int WaveMax;

  unsigned char LastBtn;
  unsigned char CurrentBtn;
   
  // Initialize pLogo
  pLogo[0]=BMP_logo0;
  pLogo[1]=BMP_logo1;
  pLogo[2]=BMP_logo2;
  pLogo[3]=BMP_logo3;
  pLogo[4]=BMP_logo4;
  pLogo[5]=BMP_logo5;
  pLogo[6]=BMP_logo6;
  pLogo[7]=BMP_logo7;

  // Initialize screen
  MGLCD.ClearScreen();

  CurrentBtn=MGLCD_BUTTON_OFF;
  do {
    LastBtn=CurrentBtn;
    
    // Get waves
    for(int i=0; i<WaveWidth; i++) {    
      wave[i]=analogRead(A0)+analogRead(A1);
    } // for i

    // Calculate WaveMin / WaveMax
    WaveMin=10000;
    WaveMax=0;  
    for(int i=0; i<WaveWidth; i++) {    
      if(wave[i]<WaveMin) WaveMin=wave[i];
      if(wave[i]>WaveMax) WaveMax=wave[i];
    } // for i
  
    // Display bar graphs
    {
      int val;
    
      val=(WaveMax-WaveMin)/divisor;
      if(val<19) {
        MGLCD.DrawBitmap(BMP_miku0,0,0,MGLCD_ROM);      
      } else if(val<24) {
        MGLCD.DrawBitmap(BMP_miku1,0,0,MGLCD_ROM);       
      } else if(val<30) {
        MGLCD.DrawBitmap(BMP_miku2,0,0,MGLCD_ROM);      
      } else if(val<28) {
        MGLCD.DrawBitmap(BMP_miku3,0,0,MGLCD_ROM);      
      } else if(val<48) {
        MGLCD.DrawBitmap(BMP_miku4,0,0,MGLCD_ROM);      
      } else if(val<60) {
        MGLCD.DrawBitmap(BMP_miku5,0,0,MGLCD_ROM);      
      } else if(val<76) {
        MGLCD.DrawBitmap(BMP_miku6,0,0,MGLCD_ROM);      
      } else if(val<96) {
        MGLCD.DrawBitmap(BMP_miku7,0,0,MGLCD_ROM);
      } else {
        MGLCD.DrawBitmap(BMP_miku8,0,0,MGLCD_ROM);            
      } // if  
    }

    // Display Logo
    {
      static int LogoLine = 0;
      static int cnt = 0;
      unsigned char bmp[LogoWidth*4+2];
    
      bmp[0]=LogoWidth;
      bmp[1]=32;
    
      for(int y=0; y<4; y++) {
        for(int x=0; x<LogoWidth; x++) {
          bmp[y*LogoWidth+x+2]=pgm_read_byte(pLogo[LogoLine%8]+(LogoLine/8+y)%(LogoHeight/8)*LogoWidth+x+2);
        } // for x
      } // for y
      MGLCD.DrawBitmap(bmp,102,0,MGLCD_RAM);
    
      if(++cnt>=4) {
        cnt=0;
        if(++LogoLine>=LogoHeight) {
          LogoLine=0;
        } // if
      } // if
    }  
    
    CurrentBtn=MGLCD.ReadButton(0);
  } while(CurrentBtn==MGLCD_BUTTON_ON || LastBtn==MGLCD_BUTTON_OFF);
} // LevelMeter4

void loop()
{
  unsigned char MeterType;
  
  // Read MeterType
  if(EEPROM.read(1)=='E') {
    MeterType=EEPROM.read(0);
    if(MeterType>=4) MeterType=0;
  } else {
    MeterType=0;
  }  
  
  // Call appropriate meter according to MeterType
  switch(MeterType) {
  case 0:
    LevelMeter1();
    break;
  case 1:
    LevelMeter2();
    break;
  case 2:
    LevelMeter3();
    break;
  case 3:
    LevelMeter4();
    break;
  } // switch
  
  // Get next MeterType
  if(++MeterType>=4) MeterType=0;
  
  // Write MeterType on EEPROM
  EEPROM.write(1,'*');
  EEPROM.write(0,MeterType);
  EEPROM.write(1,'E');
} // loop
